/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2008, Digium, Inc.
 *
 * Tilghman Lesher <format-aiff@the-tilghman.com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*! \file
 *
 * \brief Work with AIFF (ulaw internally)
 * \arg File name extension: aiff (lower case)
 * \ingroup formats
 */

/*** MODULEINFO
	<depend>sndfile</depend>
 ***/
 
#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 90158 $")

#include <sndfile.h>

#include "asterisk/mod_format.h"
#include "asterisk/module.h"

struct aiff_desc {
	SNDFILE *sf;
	SF_INFO info;
};

static int aiff_open(struct ast_filestream *s)
{
	struct aiff_desc *desc = s->_private;
	desc->info.format = SF_FORMAT_AIFF | SF_FORMAT_ULAW;

	desc->sf = sf_open_fd(fileno(s->f), s->flags & O_WRONLY ? SFM_WRITE :
			s->flags & O_RDWR ? SFM_RDWR : SFM_READ, &(desc->info), 0);

	return 0;
}

static int aiff_rewrite(struct ast_filestream *s, const char *comment)
{
	return aiff_open(s);
}

static void aiff_close(struct ast_filestream *s)
{
	struct aiff_desc *desc = s->_private;
	sf_close(desc->sf);
}

static struct ast_frame *aiff_read(struct ast_filestream *s, int *whennext)
{
	struct aiff_desc *desc = s->_private;
	loff_t res;
	int bytes = 160;
	off_t here;

	here = ftello(s->f);
	s->fr.frametype = AST_FRAME_VOICE;
	s->fr.subclass = AST_FORMAT_ULAW;
	s->fr.mallocd = 0;
	AST_FRAME_SET_BUFFER(&s->fr, s->buf, AST_FRIENDLY_OFFSET, bytes);

	if ((res = sf_read_raw(desc->sf, s->fr.data, s->fr.datalen)) <= 0) {
		if (res)
			ast_log(LOG_WARNING, "Short read (%lld) (%s)!\n", (long long)res, sf_strerror(desc->sf));
		return NULL;
	}
	s->fr.datalen = s->fr.samples = *whennext = res;
	return &s->fr;
}

static int aiff_write(struct ast_filestream *fs, struct ast_frame *f)
{
	struct aiff_desc *desc = fs->_private;
	int res;

	if (f->frametype != AST_FRAME_VOICE) {
		ast_log(LOG_WARNING, "Asked to write non-voice frame!\n");
		return -1;
	}
	if (f->subclass != AST_FORMAT_ULAW) {
		ast_log(LOG_WARNING, "Asked to write non-ULAW frame (%d)!\n", f->subclass);
		return -1;
	}
	if (!f->datalen)
		return -1;

	if ((res = sf_write_raw(desc->sf, f->data, f->datalen)) != f->datalen ) {
		ast_log(LOG_WARNING, "Bad write (%d): %s\n", res, sf_strerror(desc->sf));
		return -1;
	}

	return 0;
}

static int aiff_seek(struct ast_filestream *fs, off_t sample_offset, int whence)
{
	struct aiff_desc *desc = fs->_private;
	return sf_seek(desc->sf, sample_offset, whence) >= 0 ? 0 : -1;
}

static int aiff_trunc(struct ast_filestream *fs)
{
	int res = 0;
	aiff_close(fs);
	if (ftruncate(fileno(fs->f), ftello(fs->f)))
		res = -1;
	aiff_open(fs);
	return res;
}

static off_t aiff_tell(struct ast_filestream *fs)
{
	struct aiff_desc *desc = fs->_private;
	return sf_seek(desc->sf, 0, SEEK_CUR);
}

static const struct ast_format aiff_f = {
	.name = "aiff",
	.exts = "aiff",
	.format = AST_FORMAT_ULAW,
	.open =	aiff_open,
	.rewrite = aiff_rewrite,
	.write = aiff_write,
	.seek = aiff_seek,
	.trunc = aiff_trunc,
	.tell =	aiff_tell,
	.read = aiff_read,
	.close = aiff_close,
	.buf_size = 160 + AST_FRIENDLY_OFFSET,
	.desc_size = sizeof(SNDFILE *),
};

static int load_module(void)
{
	if (ast_format_register(&aiff_f))
		return AST_MODULE_LOAD_FAILURE;
	return AST_MODULE_LOAD_SUCCESS;
}

static int unload_module(void)
{
	return ast_format_unregister(aiff_f.name);
}	

AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "AIFF format (ulaw)");
